#import "XeeArchiveSource.h"
#import "XeeImage.h"

#import <unistd.h>

@implementation XeeArchiveSource

+(NSArray *)fileTypes
{
	return [NSArray arrayWithObjects:
		@"zip",@"rar",@"cbz",@"cbr",@"lha",@"lzh",@"7z",
	nil];
}

-(id)initWithArchive:(NSString *)archivename
{
	if(self=[super init])
	{
		archive=[[self archiveForFile:archivename] retain];
		if(!archive)
		{
			[self release];
			return nil;
		}

		tmpdir=[[NSTemporaryDirectory() stringByAppendingPathComponent:
		[NSString stringWithFormat:@"Xee-archive-%04x%04x%04x",random()&0xffff,random()&0xffff,random()&0xffff]]
		retain];

		entries=[[NSMutableDictionary dictionary] retain];

		descname=nil;

		NSArray *filetypes=[XeeImage allFileTypes];
		int count=[archive numberOfEntries];
		for(int i=0;i<count;i++)
		{
			if([archive entryIsDirectory:i]) continue;
			if([archive entryIsLink:i]) continue;

			NSString *name=[archive nameOfEntry:i];
			NSDictionary *attrs=[archive attributesOfEntry:i];
			NSString *type=NSFileTypeForHFSTypeCode([attrs fileHFSTypeCode]);
			NSString *ext=[[name pathExtension] lowercaseString];

			if([filetypes indexOfObject:ext]!=NSNotFound||[filetypes indexOfObject:type]!=NSNotFound)
			{
				NSString *realpath=[tmpdir stringByAppendingPathComponent:name];
				[entries setObject:[NSNumber numberWithInt:i] forKey:realpath];
				[self addFile:realpath sort:NO];
			}
		}

		[self sortFiles];

		[self setIcon:[[NSWorkspace sharedWorkspace] iconForFile:archivename]];
		[icon setSize:NSMakeSize(16,16)];

		[self pickImageAtIndex:0];
	}
	return self;
}

-(void)dealloc
{
	if(tmpdir) [[NSFileManager defaultManager] removeFileAtPath:tmpdir handler:nil];

	[archive release];
	[tmpdir release];
	[entries release];
	[descname release];

	[super dealloc];
}



-(NSString *)representedFilename
{
	return [archive filename];
}

-(NSString *)descriptiveNameOfCurrentImage
{
	if(!descname) return [super descriptiveNameOfCurrentImage];
	else return descname;
}

-(int)capabilities { return XeeNavigationCapable|XeeCopyingCapable; }



-(XeeFSRef *)refForPath:(NSString *)path
{
	NSNumber *entry=[entries objectForKey:path];
	if(!entry) return nil;

	int n=[entry intValue];

	[descname release];
	descname=[[archive nameOfEntry:n] retain];

	[archive _extractEntry:n as:path];

	return [XeeFSRef refForPath:path];
}



-(XADArchive *)archiveForFile:(NSString *)archivename
{
	Class archiveclass=NSClassFromString(@"XADArchive");

	if(!archiveclass)
	{
		NSString *unarchiver=[[NSWorkspace sharedWorkspace] fullPathForApplication:@"The Unarchiver"];
		if(!unarchiver)
		{
			NSString *ext=[[archivename pathExtension] lowercaseString];
			if([[XeeArchiveSource fileTypes] indexOfObject:ext]!=NSNotFound)
			{
				NSNib *nib=[[[NSNib alloc] initWithNibNamed:@"ArchiveError" bundle:nil] autorelease];
				[nib instantiateNibWithOwner:self topLevelObjects:nil];
				[errorpanel makeKeyAndOrderFront:nil];
				[self retain];
			}

			return nil;
		}

		NSString *xadpath=[unarchiver stringByAppendingPathComponent:@"Contents/Frameworks/XADMaster.framework"];
		NSBundle *xadmaster=[NSBundle bundleWithPath:xadpath];
		if(!xadmaster) return nil;
		if(![xadmaster load]) return nil;

		NSString *unipath=[unarchiver stringByAppendingPathComponent:@"Contents/Frameworks/UniversalDetector.framework"];
		NSBundle *universal=[NSBundle bundleWithPath:unipath];
		if(!universal) return nil;
		if(![universal load]) return nil;

		archiveclass=NSClassFromString(@"XADArchive");
	}

	return [archiveclass archiveForFile:archivename];
}

-(IBAction)openTheUnarchiverPage:(id)sender
{
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"http://wakaba.c3.cx/s/apps/unarchiver.html"]];
	[errorpanel close];
	[self release];
}

@end
